Package: zCompositions
Type: Package
Title: Treatment of Zeros, Left-Censored and Missing Values in
        Compositional Data Sets
Version: 1.6.1
Date: 2026-04-18
Authors@R: 
    c(person(given = "Javier",
             family = "Palarea-Albaladejo",
             role = c("cre","aut"),
             email = "javier.palarea@udg.edu",
             comment = c(ORCID = "0000-0003-0162-669X")),
      person(given = "Josep Antoni",
             family = "Martin-Fernandez",
             role = c("aut"),
             email = "josepantoni.martin@udg.edu",
             comment = c(ORCID = "0000-0003-2366-1592")))
Maintainer: Javier Palarea-Albaladejo <javier.palarea@udg.edu>
Depends: R (>= 2.14.0), methods, MASS, truncnorm, survival
ByteCompile: yes
Description: Principled methods for the imputation of zeros, left-censored and missing data in
    compositional data sets (Palarea-Albaladejo and Martin-Fernandez (2015) <doi:10.1016/j.chemolab.2015.02.019>).
License: GPL (>= 2)
Repository: CRAN
RoxygenNote: 7.3.3
Encoding: UTF-8
URL: https://github.com/Japal/zCompositions
BugReports: https://github.com/Japal/zCompositions/issues
NeedsCompilation: no
Packaged: 2026-04-18 15:21:39 UTC; Japal
Author: Javier Palarea-Albaladejo [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-0162-669X>),
  Josep Antoni Martin-Fernandez [aut] (ORCID:
    <https://orcid.org/0000-0003-2366-1592>)
Date/Publication: 2026-04-18 15:50:02 UTC
Built: R 4.6.0; ; 2026-04-25 14:54:05 UTC; unix
