import("MASS")
import("truncnorm")
import("survival")
importFrom("graphics", "lines", "abline", "axis", "barplot", "box", "grid",
     "image", "layout", "legend", "mtext", "par", "plot.new",
     "text")
importFrom("stats","sd", "approxfun", "cov", "dnorm", "pnorm", "rWishart",
     "runif", "smooth.spline", "anova", "bartlett.test", "fligner.test",
     "kruskal.test", "lm", "var","rnorm","optim", "predict", "qnorm")
importFrom("methods", "is")
importFrom("stats", "p.adjust", "pt")
importFrom("utils", "combn")
exportPattern("^[[:alpha:]]+")